<?php

/**
 * @file
 * Contains Drupal\lottiefiles_field\Plugin\Field\FieldWidget\LottieFieldWidget.
 */

namespace Drupal\lottiefiles_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\link\Plugin\Field\FieldWidget\LinkWidget;
use Drupal\Component\Utility\Crypt;

/**
 * Plugin implementation of the 'lottiefiles_field' widget.
 *
 * @FieldWidget(
 *   id = "lottiefiles_field",
 *   label = @Translation("Lottiefiles Field"),
 *   field_types = {
 *     "lottiefiles_field"
 *   }
 * )
 */
class LottiefilesFieldWidget extends LinkWidget {
  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'autoplay' => 0, // 0 = false
      'background' => 'transparent',
      'controls' => 0, // 0 = false
      'hover' => 0, // 0 = false
      'loop' => 0, // 0 = false
      'mode' => 'Normal',
      'speed' => 1,
      'selector' => '',
      'width' => 0, // 0 = false
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    // Get the field settings.
    $elements =  parent::settingsForm($form, $form_state);
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $element['uri']['#weight'] = -20;
    $randSelector = $this->generateRandomString(5);
    //@todo: Use the image style title rather than the link style title.
    /** @var \Drupal\link\LinkItemInterface $item */
    $item = $items[$delta];
    $element += [
      '#type' => 'details',
      '#open' => TRUE,
    ];
    $element += [
      'autoplay' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Autoplay'),
        '#description' => $this->t('Autoplay animation on load.'),
        '#default_value' => $item->autoplay,
      ],
      'background' => [
        '#type' => 'textfield',
        '#title' => $this->t('Background'),
        '#description' => $this->t('Background color.'),
        '#default_value' => $item->background,
      ],
      'controls' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Controls'),
        '#description' => $this->t('Show controls.'),
        '#default_value' => $item->controls,
      ],
      'hover' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Hover'),
        '#description' => $this->t('Whether to play on mouse hover.'),
        '#default_value' => $item->hover,
      ],
      'loop' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Loop'),
        '#description' => $this->t('Whether to loop animation.'),
        '#default_value' => $item->loop,
      ],
      'mode' => [
        '#type' => 'select',
        '#title' => $this->t('Mode'),
        '#options' => [
          'normal' => $this->t('Normal'),
          'bounce' => $this->t('Bounce'),
        ],
        '#description' => $this->t('Play mode.'),
        '#default_value' => $item->mode,
      ],
      'speed' => [
        '#type' => 'select',
        '#title' => $this->t('Speed'),
        '#options' => [
          1 => 1,
          2 => 2,
          3 => 3,
          4 => 4,
          5 => 5,
        ],
        '#description' => $this->t('Animation speed.'),
        '#default_value' => $item->speed,
      ],
      'selector' => [
        '#type' => 'hidden',
        '#title' => $this->t('Selector'),
        '#description' => $this->t('Unique selector identifier.'),
        '#default_value' => empty($item->selector) ? 'lottie-' . $randSelector : $item->selector,
      ],
      'width' => [
        '#type' => 'number',
        '#min' => 0,
        '#title' => $this->t('Width in pixel'),
        '#description' => $this->t('Width of the lottiefile animation.'),
        '#default_value' => empty($item->width) ? 0 : $item->width,
      ],
    ];
    return $element;
  }

  /**
   * Random String generator.
   */
  public function generateRandomString($length) {
    return Crypt::randomBytesBase64($length);
  }
}
